!
!  HexagonalHolesStructure.f90
!
!  Free-Format Fortran Source File 
!  Generated by PGI Visual Fortran(R)
!  10/5/2011 9:54:38 PM
!
module HexagonalHolesStructure

use class_HexHolesPsi
use Solvers
use class_NanoTube
use SharedTypes

contains
function GetPsi(n,m) result(psi)
	integer :: n,m
	real :: psi
	real :: Pi =  3.141592653589793
	type (PsiContainer) :: psic,psid
	real :: ZeroOfTransEq


	if (m.eq.0 ) then
		psi = Pi/n
	else
		psic = make_HexHolesPsiContainer(n,m)
		psi = NewtonRaphsonSafe(psic,0.0,Pi/n,1E-6)
	end if

end function GetPsi

function GetCosTheta(psi,n,m) result(cosTheta)
	real :: psi
	integer :: n,m
	real :: cosTheta
	real :: Pi =  3.141592653589793

	if (m .eq. 0 ) then
		cosTheta=1.0
	else 
		cosTheta=sqrt((n*(n+2*m)*sin(psi)**2)/((n+m)**2*sin(psi)**2-m**2*sin(((n+m)*psi-Pi)/m)**2))
	end if

end function GetCosTheta

function GetB(sigma,n,m,psi,cosTheta) result(b)
	real :: b
	real ::sigma,psi
	integer :: n,m
	real :: Pi =  3.141592653589793
	real :: cosTheta

	b=sigma*sqrt(1-cosTheta**2)

end function GetB

function GetRadius(tube,n,m,psi,cosTheta) result(r)
	type (NanoTube) :: tube
	real :: k,cosTheta,sinOmega,psi,r
	real :: sigma
	integer :: n,m
	real :: Pi =  3.141592653589793
	
	sigma = tube%Sigma

	if (m.eq.0) then
		r=sigma*costheta/(2*sin(psi))
	else
		r=sigma*costheta/(2*sin(psi))
	end if

end function GetRadius

subroutine GetHexHolesAtomsXYZ(tube,xyz)
	
	type (NanoTube) :: tube
	real :: k,cosTheta,sinOmega
	real ,allocatable :: xyz(:,:,:,:)

	real :: Pi =  3.141592653589793

	integer :: n,m,tubeLength
	real :: sigma,psi
	real :: r

	m=tube%M
	n=tube%N
	tubeLength=tube%Length
	sigma = tube%Sigma

	psi = GetPsi(n,m)

	cosTheta = GetCosTheta(psi,n,m)

	r = GetRadius(tube,n,m,psi,cosTheta)

	b = GetB(sigma,n,m,psi,cosTheta)

	tube%Radius = r

	if (m.eq.0) then
		call GetZigZagConfiguration(tube,psi,cosTheta,b,r,xyz)
	else
		call GetChiralAndArmchairConfiguration(tube,psi,cosTheta,b,r,xyz)
	end if

end subroutine GetHexHolesAtomsXYZ

subroutine GetChiralAndArmchairConfiguration(tube,psi,cosTheta,b,r,xyz)

	type (NanoTube) :: tube
	real :: psi,cosTheta,b,r
	real ,allocatable:: xyz(:,:,:,:)

	integer :: n,m,tubeLength
	real :: baseAtoms(3)

	integer :: atomIndex,tubeLengthIndex

	m=tube%M
	n=tube%N
	tubeLength=tube%Length
	sigma = tube%Sigma

	!!!call AllocateXYZArray(tube,xyz)

	do tubeLengthIndex=0,tubeLength
		do atomIndex=0,m-1

			baseAtoms = GetBaseAtoms(atomIndex,tubeLengthIndex,n,m,r,b,psi)
			
			xyz(1,atomIndex+1,tubeLengthIndex+1,1) = baseAtoms(1)
			xyz(1,atomIndex+1,tubeLengthIndex+1,2) = baseAtoms(2)
			xyz(1,atomIndex+1,tubeLengthIndex+1,3) = baseAtoms(3)

		end do
			
	end do

end subroutine GetChiralAndArmchairConfiguration

subroutine GetZigZagConfiguration(tube,psi,cosTheta,b,r,xyz)

	type (NanoTube) :: tube
	real :: psi,cosTheta,b,r
	real ,allocatable :: xyz(:,:,:,:)

	integer :: n,m,tubeLength
	real :: baseAtoms(3)

	integer :: atomIndex,tubeLengthIndex

	m=tube%M
	n=tube%N
	tubeLength=tube%Length
	sigma = tube%Sigma

	!!call AllocateXYZArray(tube,xyz)

	do tubeLengthIndex=0,tubeLength
		do atomIndex=0,n-1

			baseAtoms = GetBaseAtomsZigZag(atomIndex,tubeLengthIndex,r,psi,sigma)
			
			xyz(1,atomIndex+1,tubeLengthIndex+1,1) = baseAtoms(1)
			xyz(1,atomIndex+1,tubeLengthIndex+1,2) = baseAtoms(2)
			xyz(1,atomIndex+1,tubeLengthIndex+1,3) = baseAtoms(3)

		end do
			
	end do

end subroutine GetZigZagConfiguration

function GetBaseAtoms(i,t,n,m,r,b,psi) result (baseAtoms)
	implicit none
	integer :: n,m,i,t
	real :: psi,r,b
	real :: baseAtoms(3)
	real :: Pi =  3.141592653589793

	baseAtoms(1) = r*cos(2*(psi*(t+i*real(n)/real(m)-i*n/m)-i*Pi/m))
	baseAtoms(2) = r*sin(2*(psi*(t+i*real(n)/real(m)-i*n/m)-i*Pi/m))
	baseAtoms(3) = b*(t+i*real(n)/real(m)-i*n/m)

end function GetBaseAtoms

function GetBaseAtomsZigZag(i,t,r,psi,sigma) result (baseAtoms)
	implicit none
	integer :: i,t
	real :: psi,sigma,r,omega
	real :: baseAtoms(3)
	real :: Pi =  3.141592653589793

	omega=asin((1-cos(psi))/(sqrt(3.0)*sin(psi)))

	baseAtoms(1) = r*cos(psi*(2*i+t))
	baseAtoms(2) = r*sin(psi*(2*i+t))
	baseAtoms(3) = t*sigma*sin(Pi/3)*cos(omega)

end function GetBaseAtomsZigZag



end module HexagonalHolesStructure