!
!  OutputBuilder.f90
!
!  Free-Format Fortran Source File 
!  Generated by PGI Visual Fortran(R)
!  10/1/2011 9:23:28 PM
!
module OutputBuilder
use class_NanoTube
use class_NanoTubeContainer
implicit none
contains


subroutine WriteToFile(tubeContainers,fileName)
	
	character (len=100) :: fileName
	type (NanoTubeContainer) , allocatable :: tubeContainers(:)

	type (NanoTubeContainer) :: currentTubeContainer
	integer :: tubeTotal,i,TotalAtomsInFile


	tubeTotal = size(tubeContainers)

	call RelocateTubes(tubeContainers)

	TotalAtomsInFile = CountAtomsInAllTubes(tubeContainers)

	

	open(unit=20,file=trim(fileName))

		write(20,"(I0)") TotalAtomsInFile

		call PrintTubeInfoAsRemark(tubeContainers)

		do i=1,tubeTotal
			call WriteSpecificTubeToFile(tubeContainers(i))
		end do

	close(20)

	open(unit=21,file=trim(fileName) // ".info" )

		call PrintTubesTableHeaders()

		do i=1,tubeTotal
			call WriteTubesTableRow(tubeContainers(i),i)
		end do

	close(21)

end subroutine WriteToFile

subroutine WriteSpecificTubeToFile(currentTubeContainer)

	type (NanoTubeContainer)  :: currentTubeContainer
	

	real , allocatable :: xyz(:,:,:,:)	
	type (NanoTube) :: tube
	integer :: atomCount
	integer :: i,j

	tube = currentTubeContainer%Tube

	call AllocateXYZArray(tube,xyz)

	xyz = currentTubeContainer%xyz

	if (tube%M .eq. 0) then
		atomCount = tube%N
	else
		atomCount = tube%M
	end if

	  do i=1,tube%Length+1
		do j=1,atomCount

			write(20,"(A,X,F0.6,X,F0.6,X,F0.6)") trim(tube%Name),xyz(1,j,i,1),xyz(1,j,i,2),xyz(1,j,i,3)
			if (tube%TubeType .eq. 1 ) then
				write(20,"(A,X,F0.6,X,F0.6,X,F0.6)") trim(tube%Name),xyz(2,j,i,1),xyz(2,j,i,2),xyz(2,j,i,3)
			end if

		end do
	  end do
	  
	  


end subroutine WriteSpecificTubeToFile

subroutine RelocateTubes(tubeContainers)
	type (NanoTubeContainer) , allocatable :: tubeContainers(:)
	type (NanoTubeContainer)  :: currentTubeContainer

	integer :: tubeTotal
	integer :: n,m,i

	real :: moveIn 

	moveIn=0

	tubeTotal = size(tubeContainers)

	do i=1,tubeTotal
		currentTubeContainer = tubeContainers(i)
		currentTubeContainer%xyz(:,:,:,1) = currentTubeContainer%xyz(:,:,:,1) + moveIn

		moveIn = moveIn + currentTubeContainer%tube%Radius*4
		tubeContainers(i)=currentTubeContainer

	end do

end subroutine RelocateTubes

function  CountAtomsInAllTubes(tubeContainers) result (totalItemsInTubes)
	type (NanoTubeContainer) , allocatable :: tubeContainers(:)
	type (NanoTubeContainer)  :: currentTubeContainer

	integer :: tubeTotal
	integer :: i
	integer :: totalItemsInTubes

	totalItemsInTubes = 0
	tubeTotal = size(tubeContainers)

	do i=1,tubeTotal
		currentTubeContainer = tubeContainers(i)
		totalItemsInTubes = totalItemsInTubes + size(currentTubeContainer%xyz)/3
	end do

end function CountAtomsInAllTubes

subroutine PrintTubeInfoAsRemark(tubeContainers)
	type (NanoTubeContainer) , allocatable :: tubeContainers(:)

	type (NanoTube) :: tube

	
	if (size(tubeContainers) .eq. 1 ) then

		tube = tubeContainers(1)%tube
		write (20,"(A,I0,A,I0,A,A,A5,A,F4.2,A,F4.2)") "#A (" ,tube%N, "," , tube%M ,")" ," Tube material is: ",tube%Name,", radius is: ",&
tube%Radius, " bond legth used: ",tube%Sigma
	else
		write (20,"(A,I3.3)") "#multiple tubes in file total number of tubes is: ",size(tubeContainers)
	end if
end subroutine PrintTubeInfoAsRemark

subroutine PrintTubesTableHeaders()
	
	write (21,"(A6,A11,A10,A12,A10//)") "#","(n,m)   ","Element   ","R (Angstrom)","Length"

end subroutine PrintTubesTableHeaders

subroutine WriteTubesTableRow(currentTubeContainer,tubeIndex)
	type (NanoTubeContainer)  :: currentTubeContainer
	type (NanoTube) :: tube
	integer :: tubeIndex

		tube = currentTubeContainer%tube
		write (21,"(I6,A3,I2,A1,I2,A3,A10,F12.4,I10)") tubeIndex,"  (",tube%N, "," , tube%M , ")  " , tube%Name,tube%Radius,tube%length

end subroutine WriteTubesTableRow

end module OutputBuilder
