!
!  PrismicStructure.f90
!
!  Free-Format Fortran Source File 
!  Generated by PGI Visual Fortran(R)
!  10/1/2011 2:19:07 PM
!
module PrismicStructure

use class_PrismPsi
use Solvers
use class_NanoTube
use SharedTypes

contains
function GetPsi(n,m) result(psi)
	integer :: n,m
	real :: psi
	real :: Pi =  3.141592653589793
	type (PsiContainer) :: psic,psid
	real :: ZeroOfTransEq


	if (m.eq.0 ) then
		psi = Pi/n
	else
		psic = make_PrismicPsiContainer(n,m)
		psi = NewtonRaphsonSafe(psic,0.0,Pi/n,1E-6)
	end if

end function GetPsi

function GetCosTheta(psi,n,m) result(cosTheta)
	real :: psi
	integer :: n,m
	real :: cosTheta
	real :: Pi =  3.141592653589793

	cosTheta=sqrt((4*n*sin(psi)**2)/(4*n*sin(psi)**2+m*sin(((n-m)*psi-Pi)/m)**2-m*sin(((n+m)*psi-Pi)/m)**2))

end function GetCosTheta

function GetB(sigma,n,m,psi) result(b)
	real :: b
	real ::sigma,psi
	integer :: n,m
	real :: Pi =  3.141592653589793

	b=sqrt(m*sigma**2*(sin(((n-m)*psi-Pi)/m)**2-sin(((n+m)*psi-Pi)/m)**2)/ &
	(4*n*sin(psi)**2+m*(sin(((n-m)*psi-Pi)/m)**2-sin(((n+m)*psi-Pi)/m)**2)))

end function GetB

function GetRadius(tube,n,m,psi,cosTheta) result(r)
	type (NanoTube) :: tube
	real :: k,cosTheta,sinOmega,psi,r
	real :: sigma
	integer :: n,m
	real :: Pi =  3.141592653589793
	
	sigma = tube%Sigma

	if (m.eq.0) then
		r=sigma/(2*sin(psi))
	else
		r=sigma*cosTheta/(2*sin(psi))
	end if

end function GetRadius

subroutine GetPrismicAtomsXYZ(tube,xyz)
	
	type (NanoTube) :: tube
	real :: k,cosTheta,sinOmega
	real ,allocatable :: xyz(:,:,:,:)

	real :: Pi =  3.141592653589793

	integer :: n,m,tubeLength
	real :: sigma,psi
	real :: r

	m=tube%M
	n=tube%N
	tubeLength=tube%Length
	sigma = tube%Sigma

	psi = GetPsi(n,m)

	cosTheta = GetCosTheta(psi,n,m)

	r = GetRadius(tube,n,m,psi,cosTheta)

	b = GetB(sigma,n,m,psi)

	tube%Radius = r

	if (m.eq.0) then
		call GetZigZagConfiguration(tube,psi,cosTheta,b,r,xyz)
	else
		call GetChiralAndArmchairConfiguration(tube,psi,cosTheta,b,r,xyz)
	end if

end subroutine GetPrismicAtomsXYZ

subroutine GetChiralAndArmchairConfiguration(tube,psi,cosTheta,b,r,xyz)

	type (NanoTube) :: tube
	real :: psi,cosTheta,b,r
	real ,allocatable :: xyz(:,:,:,:)

	integer :: n,m,tubeLength
	real :: baseAtoms(3)

	integer :: atomIndex,tubeLengthIndex

	m=tube%M
	n=tube%N
	tubeLength=tube%Length
	sigma = tube%Sigma

	!call AllocateXYZArray(tube,xyz)

	do tubeLengthIndex=0,tubeLength
		do atomIndex=0,m-1

			baseAtoms = GetBaseAtoms(atomIndex,tubeLengthIndex,n,m,r,b,psi)
			
			xyz(1,atomIndex+1,tubeLengthIndex+1,1) = baseAtoms(1)
			xyz(1,atomIndex+1,tubeLengthIndex+1,2) = baseAtoms(2)
			xyz(1,atomIndex+1,tubeLengthIndex+1,3) = baseAtoms(3)

		end do
			
	end do

end subroutine GetChiralAndArmchairConfiguration

subroutine GetZigZagConfiguration(tube,psi,cosTheta,b,r,xyz)

	type (NanoTube) :: tube
	real :: psi,cosTheta,b,r
	real ,allocatable :: xyz(:,:,:,:)

	integer :: n,m,tubeLength
	real :: baseAtoms(3)

	integer :: atomIndex,tubeLengthIndex

	m=tube%M
	n=tube%N
	tubeLength=tube%Length
	sigma = tube%Sigma

	!call AllocateXYZArray(tube,xyz)

	do tubeLengthIndex=0,tubeLength
		do atomIndex=0,n-1

			baseAtoms = GetBaseAtomsZigZag(atomIndex,tubeLengthIndex,r,psi,sigma)
			
			xyz(1,atomIndex+1,tubeLengthIndex+1,1) = baseAtoms(1)
			xyz(1,atomIndex+1,tubeLengthIndex+1,2) = baseAtoms(2)
			xyz(1,atomIndex+1,tubeLengthIndex+1,3) = baseAtoms(3)

		end do
			
	end do

end subroutine GetZigZagConfiguration

function GetBaseAtoms(i,t,n,m,r,b,psi) result (baseAtoms)
	implicit none
	integer :: n,m,i,t
	real :: psi,r,b
	real :: baseAtoms(3)
	real :: Pi =  3.141592653589793

	integer :: nmRateR

	nmRateR = anint(real(n)/real(m))

	baseAtoms(1) = r*sin(2*(psi*(t+real(i)*real(n)/real(m)-nmRateR*i)-real(i)*Pi/real(m)))
	baseAtoms(2) = r*cos(2*(psi*(t+real(i)*real(n)/real(m)-nmRateR*i)-real(i)*Pi/real(m)))
	baseAtoms(3) = b*(t+real(i)*real(n)/real(m)-nmRateR*i)

end function GetBaseAtoms

function GetBaseAtomsZigZag(i,t,r,psi,sigma) result (baseAtoms)
	implicit none
	integer :: i,t
	real :: psi,sigma,r
	real :: baseAtoms(3)
	real :: Pi =  3.141592653589793

	baseAtoms(1) = r*cos(2*psi*i)
	baseAtoms(2) = r*sin(2*psi*i)
	baseAtoms(3) = t*sigma

end function GetBaseAtomsZigZag



end module PrismicStructure